#!/usr/bin/env python
"""
Function for deterministically generating a single hash for a directory of files,
taking into account only file contents and not filenames.

Usage:

$ dirhash('/path/to/directory', 'md5')

"""


import argparse
import checksumdir

VERSION = checksumdir.__version__


def main():
    parser = argparse.ArgumentParser(description='Determine the hash for directory.')
    parser.add_argument('-v', '--version', action='version',
                        version='checksumdir %s' % VERSION)
    parser.add_argument('directory', help='Directory for which to generate hash.')
    parser.add_argument('-a', '--algorithm', choices=('md5', 'sha1', 'sha256',
                                                      'sha512'), default='md5')
    parser.add_argument('-e', '--excluded-files', nargs='+',
                        help='List of excluded files.')
    parser.add_argument('-i', '--ignore-hidden', action='store_true', default=False,
                        help='Ignore hidden files')
    parser.add_argument('-f', '--follow-links', action='store_true', default=False,
                        help='Follow soft links')
    parser.add_argument('-x', '--excluded-extensions', nargs='+',
                        help='List of excluded file extensions.')

    args = parser.parse_args()
    print(
        checksumdir.dirhash(
            dirname=args.directory,
            hashfunc=args.algorithm,
            excluded_files=args.excluded_files,
            ignore_hidden=args.ignore_hidden,
            followlinks=args.follow_links,
            excluded_extensions=args.excluded_extensions
        )
    )

if __name__ == '__main__':
    main()



"""
Function for deterministically creating a single hash for a directory of files,
taking into account only file contents and not filenames.

Usage:

from checksumdir import dirhash

dirhash('/path/to/directory', 'md5')

"""

import os
import hashlib
import re

import pkg_resources

__version__ = pkg_resources.require("checksumdir")[0].version

HASH_FUNCS = {
    'md5': hashlib.md5,
    'sha1': hashlib.sha1,
    'sha256': hashlib.sha256,
    'sha512': hashlib.sha512
}


def dirhash(dirname, hashfunc='md5', excluded_files=None, ignore_hidden=False,
            followlinks=False, excluded_extensions=None):
    hash_func = HASH_FUNCS.get(hashfunc)
    if not hash_func:
        raise NotImplementedError('{} not implemented.'.format(hashfunc))

    if not excluded_files:
        excluded_files = []

    if not excluded_extensions:
        excluded_extensions = []

    if not os.path.isdir(dirname):
        raise TypeError('{} is not a directory.'.format(dirname))

    hashvalues = []
    for root, dirs, files in os.walk(dirname, topdown=True, followlinks=followlinks):
        if ignore_hidden and re.search(r'/\.', root):
            continue

        dirs.sort()
        files.sort()

        for fname in files:
            if ignore_hidden and fname.startswith('.'):
                continue

            if fname.split('.')[-1:][0] in excluded_extensions:
                continue

            if fname in excluded_files:
                continue

            hashvalues.append(_filehash(os.path.join(root, fname), hash_func))

    return _reduce_hash(hashvalues, hash_func)


def _filehash(filepath, hashfunc):
    hasher = hashfunc()
    blocksize = 64 * 1024

    if not os.path.exists(filepath):
        return hasher.hexdigest()

    with open(filepath, 'rb') as fp:
        while True:
            data = fp.read(blocksize)
            if not data:
                break
            hasher.update(data)
    return hasher.hexdigest()


def _reduce_hash(hashlist, hashfunc):
    hasher = hashfunc()
    for hashvalue in sorted(hashlist):
        hasher.update(hashvalue.encode('utf-8'))
    return hasher.hexdigest()



from pathlib import Path

from typing import Callable
from typing import List
from typing import Optional
from typing import Union

def dirhash(
    dirname: Union[str, Path],
    hashfunc: str='md5',
    excluded_files: Optional[List[str]]=None,
    ignore_hidden: bool=False,
    followlinks: bool=False,
    excluded_extensions: Optional[List[str]]=None
) -> str: ...

def _filehash(filepath: str, hashfunc: Callable) -> str: ...

def _reduce_hash(hashlist: List[str], hashfunc: Callable) -> str: ...

__version__: str


from setuptools import setup
import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="checksumdir",
    version="1.1.7",
    packages=['checksumdir'],
    entry_points={
        'console_scripts': ['checksumdir=checksumdir.cli:main'],
    },
    author="Tom McCarthy, M. Niedzielski",
    author_email="tmac.se@gmail.com, mark@x-powered-by.info",
    description="Compute a single hash of the file contents of a directory.",
    long_description=read('README.rst'),
    license="MIT",
    keywords="hash checksum md5 directory",
    url="http://github.com/cakepietoast/checksumdir",
    classifiers=[
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        ],
    zip_safe=False,
    package_data={"checksumdir": ["py.typed", "__init__.pyi"]}
)
